//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  Copyright (C) 2012 Werner Schweer
//  Copyright (C) 2013, 2014 Nicolas Froment, Joachim Schmitz
//  Copyright (C) 2014 Jörn Eichler
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0

MuseScore {
    version:  "1.0"
    description: "This plugin moves RH-fingering in the given range to stack with others in the chord and avoid the staff."
    menuPath: "Plugins.Notes.Fix RH-Fingering"
    
    // Apply the given function to all notes in selection
    // or, if nothing is selected, in the entire score
    
    function applyToNotesInSelection() {
        var cursor = curScore.newCursor();
        cursor.rewind(1);
        var startStaff;
        var endStaff;
        var endTick;
        var fullScore = false;
        if (!cursor.segment) { // no selection
            fullScore = true;
            startStaff = 0; // start with 1st staff
            endStaff = curScore.nstaves - 1; // and end with last
        } else {
            startStaff = cursor.staffIdx;
            cursor.rewind(2);
            if (cursor.tick == 0) {
                // this happens when the selection includes
                // the last measure of the score.
                // rewind(2) goes behind the last segment (where
                // there's none) and sets tick=0
                endTick = curScore.lastSegment.tick + 1;
            } else {
                endTick = cursor.tick;
            }
            endStaff = cursor.staffIdx;
        }
        console.log(startStaff + " - " + endStaff + " - " + endTick)
        for (var staff = startStaff; staff <= endStaff; staff++) {
            
            cursor.rewind(1); // sets voice to 0
            cursor.voice = 0; //voice has to be set after goTo
            cursor.staffIdx = staff;
            
            if (fullScore)
                cursor.rewind(0) // if no selection, beginning of score
            while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                var voice = (staff === 0 ? 3 : 0)
                var numFingers = 0
                var count = 0
                while (count < 4) {
                    cursor.voice = voice;
                    if (cursor.element && cursor.element.type === Element.CHORD) { // go through every chord
                        var notes = cursor.element.notes
                        for (var i = 0; i < notes.length; i++) {
                            var note = notes[i]
                            var noteElements = note.elements
                            for (var j = 0; j < noteElements.length; j++) {
                                console.log(noteElements[j])
                                if (noteElements[j].type === Element.FINGERING) {
									if (noteElements[j].text === "p" || noteElements[j].text === "i" || noteElements[j].text === "m" || noteElements[j].text === "a" || noteElements[j].text === "c") {
										var fingering = noteElements[j]
										console.log(fingering)
										fingering.pos.x = note.bbox.width / 2
										if (staff === 0) {
											fingering.pos.y = -1.5 -note.line/2 - numFingers*1.5
										} else {
											fingering.pos.y = 4.5 - note.line/2 + numFingers*1.5
										}
										numFingers++
									}
                                }
                            }
                        }
                    }
                    voice = (staff === 0 ? voice - 1 : voice + 1)
                    count++
                }
                cursor.voice = 0;
                cursor.next();
            }
        }
    }
    
    onRun: {
        if (typeof curScore === 'undefined')
            Qt.quit();
        
        applyToNotesInSelection()
        
        Qt.quit();
    }
}
